import { useConfig } from '@abearxiong/use-config';
import { sequelize } from '@/modules/sequelize.ts';
import { DataTypes, Model, Op } from 'sequelize';
import { createToken, checkToken } from '@abearxiong/auth/token';
import { cryptPwd } from '@abearxiong/auth';
import { nanoid } from 'nanoid';
import { CustomError } from '@abearxiong/router';
import { Org } from './org.ts';
import { redis } from '@/app.ts';

const config = useConfig<{ tokenSecret: string }>();

type UserData = {
  orgs?: string[];
};
export class User extends Model {
  declare id: string;
  declare username: string;
  declare password: string;
  declare salt: string;
  declare needChangePassword: boolean;
  declare description: string;
  declare data: UserData;
  declare type: string; // user | org | visitor
  declare owner: string;
  declare orgId: string;
  declare email: string;
  async createToken(uid?: string) {
    const { id, username, type } = this;
    const expireTime = 60 * 60 * 24 * 7; // 7 days
    const now = new Date().getTime();
    const token = await createToken({ id, username, uid, type }, config.tokenSecret);
    return { token, expireTime: now + expireTime };
  }
  static async verifyToken(token: string) {
    const ct = await checkToken(token, config.tokenSecret);
    const tokenUser = ct.payload;
    return tokenUser;
  }
  static async createUser(username: string, password?: string, description?: string) {
    const user = await User.findOne({ where: { username } });
    if (user) {
      throw new CustomError('User already exists');
    }
    const salt = nanoid(6);
    let needChangePassword = !password;
    password = password || '123456';
    const cPassword = cryptPwd(password, salt);
    return await User.create({ username, password: cPassword, description, salt, needChangePassword });
  }
  static async createOrg(username: string, owner: string, description?: string) {
    const user = await User.findOne({ where: { username } });
    if (user) {
      throw new CustomError('User already exists');
    }
    const me = await User.findByPk(owner);
    if (!me) {
      throw new CustomError('Owner not found');
    }
    if (me.type !== 'user') {
      throw new CustomError('Owner type is not user');
    }
    const org = await Org.create({ username, description, users: [{ uid: owner, role: 'owner' }] });
    const newUser = await User.create({ username, password: '', description, type: 'org', owner, orgId: org.id });
    // owner add
    await redis.del(`user:${me.id}:orgs`);
    return newUser;
  }
  createPassword(password: string) {
    const salt = this.salt;
    const cPassword = cryptPwd(password, salt);
    this.password = cPassword;
    return cPassword;
  }
  checkPassword(password: string) {
    const salt = this.salt;
    const cPassword = cryptPwd(password, salt);
    return this.password === cPassword;
  }
  async getInfo() {
    const orgs = await this.getOrgs();
    return {
      id: this.id,
      username: this.username,
      description: this.description,
      needChangePassword: this.needChangePassword,
      type: this.type,
      orgs,
    };
  }
  async getOrgs() {
    const id = this.id;
    const cache = await redis.get(`user:${id}:orgs`);
    if (cache) {
      return JSON.parse(cache);
    }
    const orgs = await Org.findAll({
      order: [['updatedAt', 'DESC']],
      where: {
        users: {
          [Op.contains]: [
            {
              uid: id,
            },
          ],
        },
      },
    });
    const orgNames = orgs.map((org) => org.username);
    await redis.set(`user:${id}:orgs`, JSON.stringify(orgNames), 'EX', 60 * 60); // 1 hour
    return orgNames;
  }
}
User.init(
  {
    id: {
      type: DataTypes.UUID,
      primaryKey: true,
      defaultValue: DataTypes.UUIDV4,
    },
    username: {
      type: DataTypes.STRING,
      allowNull: false,
      unique: true,
    },
    password: {
      type: DataTypes.STRING,
      allowNull: true,
    },
    email: {
      type: DataTypes.STRING,
      allowNull: true,
    },
    salt: {
      type: DataTypes.STRING,
      allowNull: true,
    },
    description: {
      type: DataTypes.STRING,
    },
    type: {
      type: DataTypes.STRING,
      defaultValue: 'user',
    },
    owner: {
      type: DataTypes.UUID,
    },
    orgId: {
      type: DataTypes.UUID,
    },
    needChangePassword: {
      type: DataTypes.BOOLEAN,
      defaultValue: false,
    },
    data: {
      type: DataTypes.JSON,
      defaultValue: {},
    },
  },
  {
    sequelize,
    tableName: 'cf_user',
    paranoid: true,
  },
);
User.sync({ alter: true, logging: false })
  .then((res) => {
    // initializeUser();
  })
  .catch((err) => {
    console.error('Sync User error', err);
  });

export const initializeUser = async () => {
  const w = await User.findAndCountAll();
  console.info('[User count]', w.count);
  const password = '2e8a305521bba54f49638ed25e46adf3'; //123456
  const salt = '123';
  const users = [{ username: 'admin' }, { username: 'user' }, { username: 'root' }];
  if (w.count < 1) {
    const newUsers = await User.bulkCreate(
      users.map((user) => {
        return {
          ...user,
          password,
          needChangePassword: true,
          salt,
        };
      }),
    );
    console.info('[create new Users]', newUsers);
  }
};

// initializeUser();
